#ifndef _GUI_H_
#define _GUI_H_

#define GUI_CB_NONE		0
#define GUI_CB_RETURN		1
#define GUI_CB_ALLWAYS		2
#define GUI_CB_LOST_FOCUS	3
#define GUI_CB_GET_FOCUS	4

// from 1 to 4 are free for later use
#define GUI_OBJECT_TYPE_LABEL		0
#define GUI_OBJECT_TYPE_BUTTON		5
#define GUI_OBJECT_TYPE_XBM_BUTTON	6
#define GUI_OBJECT_TYPE_INPUT		7
#define GUI_OBJECT_TYPE_HBAR		8
#define GUI_OBJECT_TYPE_VBAR		9
#define GUI_OBJECT_TYPE_MENU		10
#define GUI_OBJECT_TYPE_FLIST		11
#define GUI_OBJECT_TYPE_NLIST		12
#define GUI_OBJECT_TYPE_LIST		13
#define GUI_OBJECT_TYPE_DIAL		15
#define GUI_OBJECT_TYPE_CHECKBOX	16
#define GUI_OBJECT_TYPE_XBM		17

#define LEFT			0
#define CENTER			1
#define RIGHT			2


const struct FONT_DEF *NORMAL_FONT;
const struct FONT_DEF *BIG_FONT;
const struct FONT_DEF *SMALL_FONT;

/** *************************************************************
	@brief	Struct for a single object
	@param	label		: Text that labels an object
	@param	type		: can be : button, label, input, etc
	@param	x		: Start column
	@param	y		: Start row
	@param	w		: width
	@param	h		: height
	@param	font		: font to use
	@param	frame		: TRUE or FALSE for using a frame
	@param	status		: TRUE=active, FALSE=inactive
	@param	align		: LEFT, CENTER or RIGHT
	@param	refresh		: Used internal (don't change)
	@param	help		: Help text displayed at bottom of screen while object is active
	@param	callback	: Callback function
	@param	callback_type	: can be: on return, allways, lost or get focus
	@param	text		: Changeable text for input fields
	@param	value		: Value of dials, bar, etc
	@param	min		: Minimum value
	@param	max		: Maximum value
	@param	step_size	: Step size
	@param	xbm		: Pointer to xbm data
*****************************************************************/
typedef struct
{
  // For all kind of object
  char label[MAX_GUI_LABEL_SIZE];
  int type;
  int x;
  int y;
  int w;
  int h;
  const struct FONT_DEF *font;
  int frame;
  int status;
  int refresh;
  char help[MAX_GUI_LABEL_SIZE];
  // Only for active objects
  uint8_t (*callback)(void);
  int callback_type;
  // for text inputs only
  char text[MAX_GUI_TEXT_SIZE];
  // for integer inputs, like bars, dials, etc.
  int16_t value;
  int16_t min;
  int16_t max;
  int16_t step_size;
  // lists
  int list_active;
  uint8_t num_elements;
  PGM_P *elements;
  uint8_t first_element;
  int listheight;
  // for pictures
  const unsigned char *xbm;
}GUI_OBJECT;

/** *************************************************************
	@brief	Struct for a dialog
	@param	x		: Start column
	@param	y		: start row
	@param	w		: width
	@param	h		: height
	@param	font		: The font for the titelbar
	@param	active_object	: The number of the active object
	@param	num_objects	: Counter for objects
	@param	object		: Pointer array to objects
*****************************************************************/
typedef struct
{
  int active_object;
  int num_objects;
  GUI_OBJECT *object[MAX_GUI_OBJECTS];
}GUI_DIALOG;

GUI_DIALOG GUI_ACTIVE_DIALOG;					// This holds the active dialog for referenzing

void GUI_SET_SMALL_FONT(const struct FONT_DEF *font);
void GUI_SET_BIG_FONT(const struct FONT_DEF *font);
void GUI_SET_NORMAL_FONT(const struct FONT_DEF *font);

GUI_OBJECT GUI_OBJECT_NEW(int type, int x, int y, int w, int h, char *label);
GUI_DIALOG GUI_DIALOG_NEW(void);

void GUI_OBJECT_SET_HELP(GUI_OBJECT *object, char *text);
void GUI_OBJECT_SET_HELP_p(GUI_OBJECT *object, char *text);
#define GUI_OBJECT_SET_HELP_P(object, text) GUI_OBJECT_SET_HELP_p(object,PSTR(text))
void GUI_OBJECT_SET_LABEL(GUI_OBJECT *object, char *text);
void GUI_OBJECT_SET_TEXT(GUI_OBJECT *object, char *text);
void GUI_OBJECT_SET_POSITION(GUI_OBJECT *object, int x, int y, int w, int h);
void GUI_OBJECT_SET_FRAME(GUI_OBJECT *object, int frame);
void GUI_OBJECT_SET_CALLBACK(GUI_OBJECT *object, uint8_t (*cb)(void), int callback_type);
void GUI_OBJECT_SET_VALUES(GUI_OBJECT *object, int16_t value, int16_t min, int16_t max, int16_t step_size);
void GUI_OBJECT_SET_VALUE_UP(GUI_OBJECT *object);
void GUI_OBJECT_SET_VALUE_DOWN(GUI_OBJECT *object);
void GUI_OBJECT_SET_VALUE(GUI_OBJECT *object, uint16_t value);
void GUI_OBJECT_SET_XBM(GUI_OBJECT *object, const unsigned char *xbm);
void GUI_OBJECT_SET_FONT(GUI_OBJECT *object,  const struct FONT_DEF *font);
char *GUI_OBJECT_GET_TEXT(GUI_OBJECT *object);
int GUI_OBJECT_GET_VALUE(GUI_OBJECT *object);
void GUI_OBJECT_REFRESH(GUI_OBJECT *object);

void GUI_OBJECT_INSERT_LIST_P(GUI_OBJECT *object, PGM_P *elements, uint8_t num);
void GUI_OBJECT_INSERT_NLIST(GUI_OBJECT *object, uint8_t min, uint8_t max);
void GUI_OBJECT_LIST_SET_COUNT(GUI_OBJECT *object, uint8_t num);
uint8_t GUI_OBJECT_LIST_GET_INDEX(GUI_OBJECT *object);
void GUI_OBJECT_INSERT_LIST(GUI_OBJECT *object, uint8_t num);

void GUI_DIALOG_INSERT_OBJECT(GUI_DIALOG *dialog, GUI_OBJECT *object);

void GUI_DIALOG_DISPLAY(GUI_DIALOG *dialog);
void GUI_OBJECT_DISPLAY(GUI_OBJECT *object);
void GUI_DIALOG_CLOSE(GUI_DIALOG *dialog);
void GUI_DIALOG_SET_ACTIVE_DIALOG(GUI_DIALOG *dialog);

void GUI_RUN(void);
#endif
