#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <alloca.h>
#include <math.h>
#include "include/general.h"
#include "include/config.h"
#include "include/t6963c.h"
#include "include/graphics.h"
#include "include/inputs.h"
#include "include/font.h"
#include "include/gui.h"
#include "include/timer.h"



/** *************************************************************
	@brief	Set the small font
	@param 	font	: The font
	@return	none
*****************************************************************/
void GUI_SET_SMALL_FONT(const struct FONT_DEF *font)
{
  SMALL_FONT=font;
}

/** *************************************************************
	@brief	Set the standard font 
	@param 	font	: The font
	@return	none
*****************************************************************/
void GUI_SET_NORMAL_FONT(const struct FONT_DEF *font)
{
  NORMAL_FONT=font;
}

/** *************************************************************
	@brief	Set the big font 
	@param 	font	: The font
	@return	none
*****************************************************************/
void GUI_SET_BIG_FONT(const struct FONT_DEF *font)
{
  BIG_FONT=font;
}

/** *************************************************************
	@brief	Create a new object
	@param 	type	: The type of the object
	@param	x	: Start column
	@param	y	: Start row
	@param	w	: Width
	@param	h	: Height
	@param	label	: The label
	@return		: Pointer to the new object
*****************************************************************/
GUI_OBJECT GUI_OBJECT_NEW(int type, int x, int y, int w, int h, char *label)
{
  GUI_OBJECT *object = (GUI_OBJECT *)alloca (sizeof (GUI_DIALOG));
  object->type=type;
  object->x=x;
  object->y=y;
  object->w=w;
  object->h=h;
  object->status=FALSE;
  object->refresh=FALSE;
  sprintf(object->label,label);
  object->value=FALSE;
  object->callback_type=GUI_CB_NONE;
  object->font=NORMAL_FONT;
  return *object;
}


/** *************************************************************
	@brief	Create a new dialog
	@param	x	: Start column
	@param	y	: Start row
	@param	w	: Width
	@param	h	: Height
	@return		: Pointer to the new dialog
*****************************************************************/
GUI_DIALOG GUI_DIALOG_NEW(void)
{
  GUI_DIALOG *dialog = (GUI_DIALOG*)alloca (sizeof (GUI_DIALOG));
  dialog->num_objects=0;
  dialog->active_object=0;
  return *dialog;
}

/** *************************************************************
	@brief	Insert a list from flash into the list
	@param 	object 	: The object
	@param 	element	: The list
	@param	num	: How many entrys
	@return	none
*****************************************************************/
void GUI_OBJECT_INSERT_LIST_P(GUI_OBJECT *object, PGM_P *elements, uint8_t num)
{
    object->elements=elements;
    object->num_elements=num;
    object->list_active=0;
    object->first_element=0;
    object->listheight=object->h/GLCD_GET_FONT_HEIGHT(NORMAL_FONT);
}

/** *************************************************************
	@brief	Insert a list from ram into the list
	@param 	object 	: The object
	@param 	element	: The list
	@param	num	: How many entrys
	@return	none
*****************************************************************/
void GUI_OBJECT_INSERT_LIST(GUI_OBJECT *object, uint8_t num)
{
    object->num_elements=num;
    object->list_active=0;
    object->first_element=0;
    object->listheight=object->h/GLCD_GET_FONT_HEIGHT(NORMAL_FONT);
}

/** *************************************************************
	@brief	Set values of a number list
	@param 	object 	: The object
	@param 	min	: first number
	@param	max	: last number
	@return	none
*****************************************************************/
void GUI_OBJECT_INSERT_NLIST(GUI_OBJECT *object, uint8_t min, uint8_t max)
{
    object->num_elements=max-min;
    object->list_active=min;
    object->first_element=min;
    object->min=min;
    object->max=max;
    object->listheight=object->h/GLCD_GET_FONT_HEIGHT(NORMAL_FONT);
}

/** *************************************************************
	@brief	Set number of elements in the list
	@param 	object 	: The object
	@param 	num	: The count
	@return	none
*****************************************************************/
void GUI_OBJECT_LIST_SET_COUNT(GUI_OBJECT *object, uint8_t num)
{
    object->num_elements=num;
}

/** *************************************************************
	@brief	Get the index of last active list element
	@param 	object 	: The object
	@return	the index number
*****************************************************************/
uint8_t GUI_OBJECT_LIST_GET_INDEX(GUI_OBJECT *object)
{
  return object->list_active;
}

/** *************************************************************
	@brief	Set help text for an object
	@param 	object 	: The object
	@param 	text	: The text
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_HELP(GUI_OBJECT *object, char *text)
{
  sprintf(object->help,text);
}

/** *************************************************************
	@brief	Set help text for an object from flash
	@param 	object 	: The object
	@param 	text	: The text
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_HELP_p(GUI_OBJECT *object, char *text)
{
  char m;
  int x=0;
  while((m = pgm_read_byte(text++))) 
  {
    object->help[x]=m;
    x++;
  }
  object->help[x]='\0';
}

/** *************************************************************
	@brief	Set a new label for an object
	@param 	object : The object
	@param 	text : The text to set
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_LABEL(GUI_OBJECT *object, char *text)
{
  sprintf(object->label,text);
}

/** *************************************************************
	@brief	Set the font of an label
	@param 	object : The object
	@param 	font : The font to set
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_FONT(GUI_OBJECT *object,  const struct FONT_DEF *font)
{
  object->font=font;
}

/** *************************************************************
	@brief	Set the text for an input object
	@param 	object : The object
	@param 	text : The text to set
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_TEXT(GUI_OBJECT *object, char *text)
{
  sprintf(object->text,text);
}

/** *************************************************************
	@brief	Set position and size of an object
	@param 	object : The object
	@param 	x : Start column
	@param 	y : Start row
	@param 	w : Width
	@param 	h : Height
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_POSITION(GUI_OBJECT *object, int x, int y, int w, int h)
{
  object->x=x;
  object->y=y;
  object->w=w;
  object->h=h;
}

/** *************************************************************
	@brief	Set the frame type of an input object
	@param 	object : The object
	@param 	frame : TRUE or FALSE
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_FRAME(GUI_OBJECT *object, int frame)
{
  object->frame=frame;
}

/** *************************************************************
	@brief	Set the callback of an input object
	@param 	object 		: The object
	@param 	callback 	: The function
	@param	callback_type	: GUI_CB_RETURN, GUI_CB,ALLWAYS, GUI_CB_LOST_FOCUS, GUI_CB_GET_FOCUS
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_CALLBACK(GUI_OBJECT *object, uint8_t (*cb)(void), int callback_type)
{
  object->callback=cb;
  object->callback_type=callback_type;
}

/** *************************************************************
	@brief	Set the value of an object like slider, bar, dial, etc
	@param 	object		: The object
	@param	value		; The real value
	@param	min		: minimum value
	@param	max		: maximum value
	@param	step_size	: Size of a step
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_VALUES(GUI_OBJECT *object, int16_t value, int16_t min, int16_t max, int16_t step_size)
{
  object->value=value;
  object->min=min;
  object->max=max;
  object->step_size=step_size;
}

/** *************************************************************
	@brief	Set the value +step_size of an object
	@param 	object		: The object
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_VALUE_UP(GUI_OBJECT *object)
{
  object->value+=object->step_size;
  if(object->value>=object->max)
  {
    object->value=object->max;
  }
}

/** *************************************************************
	@brief	Set the value -step_size of an object
	@param 	object		: The object
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_VALUE_DOWN(GUI_OBJECT *object)
{
  object->value-=object->step_size;
  if(object->value<=object->min)
  {
    object->value=object->min;
  }
}

/** *************************************************************
	@brief	Set the value of an object
	@param 	object		: The object
	@param	value		: The value to set
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_VALUE(GUI_OBJECT *object, uint16_t value)
{
  object->value=value;
  if(object->type!=GUI_OBJECT_TYPE_CHECKBOX)
  {
    if(object->value>=object->max)
    {
      object->value=object->max;
    }
    if(object->value<=object->min)
    {
      object->value=object->min;
    }
  }
}

/** *************************************************************
	@brief	Set the xbm picture of an input object
	@param 	object : The object
	@param 	xbm : picture data in flash
	@return	none
*****************************************************************/
void GUI_OBJECT_SET_XBM(GUI_OBJECT *object, const unsigned char *xbm)
{
  object->xbm=xbm;
}

/** *************************************************************
	@brief	Get the text of an input object
	@param 	object : The object
	@return	The text
*****************************************************************/
char *GUI_OBJECT_GET_TEXT(GUI_OBJECT *object)
{
  return object->text;
}

/** *************************************************************
	@brief	Get the value of an object like slider, dial, bar, etc
	@param 	object : The object
	@return	The value
*****************************************************************/
int GUI_OBJECT_GET_VALUE(GUI_OBJECT *object)
{
  return object->value;
}

/** *************************************************************
	@brief	Insert an object into a dialog
	@param 	dialog : The dialog to insert into
	@param 	object : The object to insert
	@return	none
*****************************************************************/
void GUI_DIALOG_INSERT_OBJECT(GUI_DIALOG *dialog, GUI_OBJECT *object)
{
  dialog->object[dialog->num_objects]=object;
  dialog->num_objects++;
}

/** *************************************************************
	@brief	Refresh an object on screen
	@param 	object : The object
	@return	none
*****************************************************************/
void GUI_OBJECT_REFRESH(GUI_OBJECT *object)
{
  object->refresh=FALSE;				// Set object on refresh so that the background will not be cleared
  GUI_OBJECT_DISPLAY(object);			// Redisplay it
}
/** *************************************************************
	@brief	Displays an object
		If calling this function from program, don't 
		forget, that it's referenzing the active dialog
		for placement.
	@param 	object	: The object to show
	@param	state	: TRUE=Active or FALSE=Inactive
	@return	none
*****************************************************************/
void GUI_OBJECT_DISPLAY(GUI_OBJECT *object)
{
  char listtemp[MAX_GUI_LABEL_SIZE];
  uint8_t i=0;
  int p=0;
  // For calculation of real position inside dialog
  int x=object->x;		// The real x
  int y=object->y;		// The real y
  
  int w=x + object->w;			// The real w position
  int h=y + object->h;			// The real h position
 
  int xa=0;
  //Draw text, paint pictures, etc
  switch (object->type)
  {
    case GUI_OBJECT_TYPE_LABEL:
      xa=1;
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w, h, FALSE);
      }
      GLCD_PRINT(x+xa , y , object->label, object->font, object->status);
      // If frame is set, draw it. 
      if(object->frame==TRUE)
      {
	GLCD_BOX(x , y , w, h, TRUE);
      }
      break;
    case GUI_OBJECT_TYPE_BUTTON:
      xa=((w-x-1)/2)-(GLCD_STRLEN(object->label,NORMAL_FONT)/2)+1;
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w, h, FALSE);
      }
      if(object->status==TRUE)
      {
	GLCD_BOX_FILLED(x , y , w, h-2, TRUE);
      }
      GLCD_PRINT(x+xa , y , object->label, NORMAL_FONT, object->status);
      GLCD_BOX(x , y , w, h-2, TRUE);		// Frame around button
      GLCD_LINE(x+1, h-1, w, h-1, TRUE);	// Bottom shadow
      GLCD_LINE(w+1, y+1, w+1, h-1, TRUE);	// Right shadow
      break;
    case GUI_OBJECT_TYPE_XBM_BUTTON:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w, h, FALSE);
      }
      GLCD_XBM(object->xbm, x, y, object->w, object->h, SOLID);
      if(object->status==TRUE)
      {
	GLCD_BOX(x , y , w, h, TRUE);
      }
      break;
    case GUI_OBJECT_TYPE_XBM:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w, h, FALSE);
      }
      GLCD_XBM(object->xbm, x, y, object->w, object->h, SOLID);
      break;
    case GUI_OBJECT_TYPE_HBAR:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w, h, FALSE);
      }
      GLCD_BOX_FILLED(x+2, y+2, x+((object->w-4)*object->value/object->max)+2, h-2, TRUE);
      GLCD_BOX_FILLED(x+((object->w-4)*object->value/object->max)+3, y+3, w, h-3, FALSE);
      GLCD_BOX(x+2, y+2, w-2, h-2, TRUE);
      sprintf(object->help,"Value : %3d  ",object->value);
      if(object->status==TRUE)
      {
	GLCD_BOX(x , y , w, h, TRUE);
      }
      break;
    case GUI_OBJECT_TYPE_VBAR:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w, h, FALSE);
      }
      GLCD_BOX_FILLED(x+2, y, w-2, h-((object->h-2)*object->value/object->max), FALSE);
      GLCD_BOX_FILLED(x+3, h-((object->h-2)*object->value/object->max), w-2, h-3, TRUE);
      GLCD_BOX(x+2, y+2, w-2, h-2, TRUE);
      sprintf(object->help,"Value : %3d  ",object->value);
      if(object->status==TRUE)
      {
	GLCD_BOX(x , y , w, h, TRUE);
      }
      break;
    case GUI_OBJECT_TYPE_CHECKBOX:
      xa=1;
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x , y , w+10, h, FALSE);
      }
      GLCD_PRINT(x+xa+10 , y , object->label, NORMAL_FONT, FALSE);
      if(object->value==FALSE)
      {
	GLCD_BOX(x+2, y+2, x+10, y+12, TRUE);
      }
      else if(object->value==TRUE)
      {
	GLCD_BOX(x+2, y+2, x+10, y+12, TRUE);
	GLCD_LINE(x+4, y+8, x+6, y+10, TRUE);
	GLCD_LINE(x+6, y+8, x+8, y+4, TRUE);
      }
      if(object->status==TRUE)
      {
	GLCD_BOX(x , y , w+10, h, TRUE);
      }
//      GLCD_BOX(x+2 , y+2 , w+GLCD_STRLEN("(X) ",object->font)-2, h-2, TRUE);		// Frame around button
      break;
    case  GUI_OBJECT_TYPE_FLIST:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x+1 , y+1 , w-1, h-1, FALSE);
      }
      for(i=object->first_element;i<=object->first_element+object->listheight;i++)
      {
	strcpy_P(listtemp, (PGM_P)pgm_read_word(&(object->elements[i])));
	if(i==object->list_active)
	{
	  GLCD_PRINT(x , y+(GLCD_GET_FONT_HEIGHT(NORMAL_FONT)*p) , listtemp, NORMAL_FONT, TRUE);
	}
	else
	{
	  GLCD_PRINT(x , y+(GLCD_GET_FONT_HEIGHT(NORMAL_FONT)*p) , listtemp, NORMAL_FONT, FALSE);
	}
	p++;
      }
      // Draw the frame
      GLCD_BOX(x , y , w, h, TRUE);

      break;
    case GUI_OBJECT_TYPE_NLIST:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x+1 , y+1 , w-1, h-1, FALSE);
      }
      for(i=object->first_element;i<=object->first_element+object->listheight;i++)
      {
	sprintf(listtemp,"%3d",i);
	if(i==object->list_active)
	{
	  GLCD_PRINT(x , y+(GLCD_GET_FONT_HEIGHT(NORMAL_FONT)*p) , listtemp, NORMAL_FONT, TRUE);
	}
	else
	{
	  GLCD_PRINT(x , y+(GLCD_GET_FONT_HEIGHT(NORMAL_FONT)*p) , listtemp, NORMAL_FONT, FALSE);
	}
	p++;
      }
      // Draw the frame
      GLCD_BOX(x , y , w, h, TRUE);

      break;
    case  GUI_OBJECT_TYPE_LIST:
      // Delete the background only if not on refresh
      if(object->refresh==FALSE)
      {
	GLCD_BOX_FILLED(x+1 , y+1 , w-1, h-1, FALSE);
      }
      for(i=object->first_element ; i<=object->first_element + object->listheight ; i++)
      {
	sprintf(listtemp, ram_list[i]);
	if(i==object->list_active)
	{
	  GLCD_PRINT(x , y+(GLCD_GET_FONT_HEIGHT(NORMAL_FONT)*p) , listtemp, NORMAL_FONT, TRUE);
	}
	else
	{
	  GLCD_PRINT(x , y+(GLCD_GET_FONT_HEIGHT(NORMAL_FONT)*p) , listtemp, NORMAL_FONT, FALSE);
	}
	p++;
      }
      // Draw the frame
      GLCD_BOX(x , y , w, h, TRUE);

      break;
  }

  if( (object->refresh==FALSE))
  {
    GLCD_BOX_FILLED(0,GLCD_YMAX-GLCD_GET_FONT_HEIGHT(NORMAL_FONT),GLCD_XMAX,GLCD_YMAX,TRUE);
  }
  // if a help text is avaiable, then show it at bottom of screen
  if((strlenght(object->help) > 0) && (object->status==TRUE) )
  {
    GLCD_BOX_FILLED(0,GLCD_YMAX-GLCD_GET_FONT_HEIGHT(NORMAL_FONT),GLCD_XMAX,GLCD_YMAX,TRUE);
    GLCD_PRINT(0,GLCD_YMAX-GLCD_GET_FONT_HEIGHT(NORMAL_FONT),object->help,NORMAL_FONT,TRUE);
  }
  object->refresh=FALSE;				// Set refresh of object to false
}

/** *************************************************************
	@brief	Displays a dialog
	@param 	dialog : The dialog to show
	@return	none
*****************************************************************/
void GUI_DIALOG_DISPLAY(GUI_DIALOG *dialog)
{
  int i=0;
  int has_active=FALSE;

  // Close running dialog if there is one
  GUI_DIALOG_CLOSE(&GUI_ACTIVE_DIALOG);
  GLCD_CLEAR();
//  GLCD_LINE(0,GLCD_YMAX-GLCD_GET_FONT_HEIGHT(GUI_STANDARD_FONT)+3,GLCD_XMAX,GLCD_YMAX-GLCD_GET_FONT_HEIGHT(GUI_STANDARD_FONT)+3,TRUE);
  GUI_ACTIVE_DIALOG=*dialog;
  for(i=0;i<dialog->num_objects;i++)
  {
    // If we have no active object we set the next useable
      if(has_active==FALSE)
      {
	if ( (dialog->object[i]->type==GUI_OBJECT_TYPE_BUTTON) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_XBM_BUTTON) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_INPUT) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_HBAR) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_VBAR) ||
	       (dialog->object[i]->type== GUI_OBJECT_TYPE_FLIST) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_NLIST) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_LIST) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_CHECKBOX) ||
	       (dialog->object[i]->type==GUI_OBJECT_TYPE_DIAL) )
	{
	  dialog->object[i]->status=TRUE;
	  dialog->active_object=i;
	  GUI_ACTIVE_DIALOG.active_object=i;
	  has_active=TRUE;
	}
      }
      dialog->object[i]->refresh=TRUE;
      GUI_OBJECT_DISPLAY(dialog->object[i]);
  }
}

/** *************************************************************
	@brief	Close a dialog
	@param 	dialog 	: The dialog to show
	@param	f	: TRUE=free dialog mem
	@return	none
*****************************************************************/
void GUI_DIALOG_CLOSE(GUI_DIALOG *dialog)
{
/*
  int i=0;
  
    for(i=0;i<dialog->num_objects;i++)
    {
      if(dialog->object)
      {
	free(dialog->object[i]);
      }
    }
*/
    if(dialog)
    {
      free(dialog);
    }
}

/** *************************************************************
	@brief	Makes a dialog to the active one
	@param 	dialog 	: The dialog 
	@return	none
*****************************************************************/
void GUI_DIALOG_SET_ACTIVE_DIALOG(GUI_DIALOG *dialog)
{
  GUI_ACTIVE_DIALOG=*dialog;
}

/** *************************************************************
	@brief	Handles the GUI System
	@param 	none
	@return	none
*****************************************************************/
void GUI_RUN(void)
{
  uint16_t old_value=0;
  uint16_t value=0;
  int running=TRUE;
  
  while(running==TRUE)
  {
    if( GET_KEY( KEY_ENTER ))
    {
      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback_type!=GUI_CB_NONE)			// Only if object has a callback
      {
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback();					// Invoce the callback
      }
      // Checkboxes need toggle of check
      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type==GUI_OBJECT_TYPE_CHECKBOX)
      {
	if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->value==TRUE)
	{
	  GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->value=FALSE;
	  GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);				// Redisplay it
	}
	else// if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->value==FALSE)
	{
	  GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->value=TRUE;
	  GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);				// Redisplay it
	}
      }
      old_value=0;
    }
 
    if( GET_KEY( KEY_MENU ))
    {
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->status=FALSE;				// Set object status to false
	GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);				// Redisplay it
	GUI_ACTIVE_DIALOG.active_object++;	// Set the next object in dialog active
	while( (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_BUTTON) &&	// Search for the next active object type
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_XBM_BUTTON) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_INPUT) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_HBAR) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_VBAR) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!= GUI_OBJECT_TYPE_FLIST) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_NLIST) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_LIST) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_CHECKBOX) &&
	       (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type!=GUI_OBJECT_TYPE_DIAL) )
	{
	  GUI_ACTIVE_DIALOG.active_object++;									// Count to next object
	  if(GUI_ACTIVE_DIALOG.active_object>=GUI_ACTIVE_DIALOG.num_objects)		
	  {
	    GUI_ACTIVE_DIALOG.active_object=0;									// Don't get over object count
	  }
	}
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->status=TRUE;					// Set status of new object to true
	GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);				// Redisplay it
	old_value=0;
    }
    
    if( GET_KEY( KEY_UP ))
    {
      switch (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type)
      {
	case GUI_OBJECT_TYPE_HBAR:
	case GUI_OBJECT_TYPE_VBAR:
	  GUI_OBJECT_SET_VALUE_UP(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Count value up
	  GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->refresh=TRUE;				// Set object on refresh so that the background will not be cleared
	  GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
	case  GUI_OBJECT_TYPE_FLIST:
	  if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active < GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->num_elements-1)	// if not at the end of list
	  {
	    GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active++;				// count active element up
	  }
	    if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active > GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element+GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->listheight)	// if not at the end of elements+2
	    {
	      GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element++;			// count first element up
	    }
	    GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
	case GUI_OBJECT_TYPE_NLIST:
	  if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active < GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->max)	// if not at the end of list
	  {
	    GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active++;				// count active element up
	  }
	    if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active > GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element+GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->listheight)	// if not at the end of elements+2
	    {
	      GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element++;			// count first element up
	    }
	    GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
	case  GUI_OBJECT_TYPE_LIST:
	  if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active < GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->num_elements-1)	// if not at the end of list
	  {
	    GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active++;				// count active element up
	  }
	    if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active > GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element+GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->listheight)	// if not at the end of elements+2
	    {
	      GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element++;			// count first element up
	    }
	    GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
      }
      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback_type==GUI_CB_ALLWAYS)
      {
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback();					// Invoce the callback
      }
    }
 
    if( GET_KEY( KEY_DOWN ))
    {
      switch (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type)
      {
	case GUI_OBJECT_TYPE_HBAR:
	case GUI_OBJECT_TYPE_VBAR:
	  GUI_OBJECT_SET_VALUE_DOWN(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// count value down
	  GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->refresh=TRUE;				// Set object on refresh
	  GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	case  GUI_OBJECT_TYPE_FLIST:
	    if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active > 0)			// if the beginning of list is not reached
	    {
	      GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active--;				// Count active element down
	    }
	      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active < GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element)	// if we not at the beginning of elements
	      {
		GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element--;			// count the first element to get down
	      }
	      GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
	case GUI_OBJECT_TYPE_NLIST:
	    if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active > GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->min)			// if the beginning of list is not reached
	    {
	      GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active--;				// Count active element down
	    }
	      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active < GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element)	// if we not at the beginning of elements
	      {
		GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element--;			// count the first element to get down
	      }
	      GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
	case  GUI_OBJECT_TYPE_LIST:
	    if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active > 0)			// if the beginning of list is not reached
	    {
	      GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active--;				// Count active element down
	    }
	      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->list_active < GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element)	// if we not at the beginning of elements
	      {
		GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->first_element--;			// count the first element to get down
	      }
	      GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Redisplay it
	  break;
      }
      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback_type==GUI_CB_ALLWAYS)
      {
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback();					// Invoce the callback
      }
    }
    // We handle the bars with the encoder switch
    if( (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type==GUI_OBJECT_TYPE_HBAR) || (GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->type==GUI_OBJECT_TYPE_VBAR))
    {
      value=GUI_OBJECT_GET_VALUE(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);			// Get the value of the object
      value+=ENCODE_READ();											// Add the encoder value
      if(old_value!=value)											// If old and new value differend
      {
	GUI_OBJECT_SET_VALUE(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object],value);			// Set the new object value
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->refresh=TRUE;					// Set object to refresh
	GUI_OBJECT_DISPLAY(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]);				// Redisplay the object
	old_value=value;												// Save value as old value
      }
      if(GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback_type==GUI_CB_ALLWAYS)
      {
	GUI_ACTIVE_DIALOG.object[GUI_ACTIVE_DIALOG.active_object]->callback();					// Invoce the callback
      }
    }
    
    if( f_timer_tick ){
      f_timer_tick = 0;
      GUI_TIMER_TICK();
    }
    
  }
}
