#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "include/config.h"
#include "include/inputs.h"
#include "include/gui.h"
#include "include/t6963c.h"
#include "include/font.h"


/** *************************************************************
	@brief	Table and delta variable for rotary switch
*****************************************************************/
volatile int8_t enc_delta;
int8_t table[16] PROGMEM = {0,0,-1,0,0,0,0,1,1,0,0,0,0,-1,0,0};    //halb

 
/** *************************************************************
	@brief	Variables for key handling
*****************************************************************/
volatile uint8_t key_state;                               // debounced and inverted key state:
							  // bit = 1: key pressed
volatile uint8_t key_press;                               // key press detect
 
 
/** *************************************************************
	@brief	Timer compare interrupt for inputs
*****************************************************************/
ISR( TIMER2_COMPA_vect )
{
  static char ct0, ct1;
  char i;
// Rotary switch
static int8_t last=0;           // Store old value
 
  TCNT0 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);  // preload for 10ms
 
  i = key_state ^ ~KEY_PIN;		// key changed ?
  ct0 = ~( ct0 & i );			// reset or count ct0
  ct1 = (ct0 ^ ct1) & i;			// reset or count ct1
  i &= ct0 & ct1;			// count until roll over ?
  key_state ^= i;			// then toggle debounced state
  key_press |= key_state & i;		// 0->1: key press detect

  f_timer_tick = 1;

// Rotary switch
 last = (last << 2)  & 0x0F;
 if (PHASE_A) last |=2;
 if (PHASE_B) last |=1;
 enc_delta += pgm_read_byte(&table[last]);
    

}
 
/** *************************************************************
	@brief	Check if key has been pressed
	@param 	key_mask : The bit(s) of the key(s)
	@return	1 if key was pressed
*****************************************************************/
uint8_t GET_KEY_PRESS( uint8_t key_mask )
{
  cli();                                          // read and clear atomic !
  key_mask &= key_press;                          // read key(s)
  key_press ^= key_mask;                          // clear key(s)
  sei();
  return key_mask;
}
 
/** *************************************************************
	@brief	For user check if key was pressed
	@param 	key_mask : The bit(s) of the key(s)
	@return	1 if key was pressed
*****************************************************************/
uint8_t GET_KEY_( uint8_t key_mask )
{
  return GET_KEY_PRESS(  key_mask );
}
 
/** *************************************************************
	@brief	Init the timer and ports for keys and switch
	@param 	none
	@return	none
*****************************************************************/
void INIT_INPUTS( void )
{
  // Configure debouncing routines
  KEY_DDR &= ~ALL_KEYS;                // configure key port for input
  KEY_PORT |= ALL_KEYS;                // and turn on pull up resistors
 
  TCNT2=255;
  TCCR2A |= (1<<WGM21);
  TCCR2B |= (1<<CS22);// | (1<<CS20);
  TIMSK2 |= (1<<OCIE2A);
  OCR2A=255;
}

/** *************************************************************
	@brief	Read single step encoder
	@param 	none
	@return	value from table
*****************************************************************/
int8_t ENCODE_READ( void )
{
  int8_t val;
  cli();
  val = enc_delta;
  enc_delta = 0;
  sei();
  return val;                   // counts since last call
}
 
 
/** *************************************************************
	@brief	Read two step encoder
	@param 	none
	@return	value from table
*****************************************************************/
int8_t ENCODE_READ_2( void )
{
  int8_t val;
 
  cli();
  val = enc_delta;
  enc_delta = val & 1;
  sei();
  return val >> 1;
}
 
 
/** *************************************************************
	@brief	Read four step encoder
	@param 	none
	@return	value from table
*****************************************************************/
int8_t ENCODE_READ_4( void )
{
  int8_t val;
 
  cli();
  val = enc_delta;
  enc_delta = val & 3;
  sei();
  return val >> 2;
}

